searchData={"items":[{"type":"module","title":"PhoenixTest","doc":"PhoenixTest provides a unified way of writing feature tests -- regardless of\n  whether you're testing LiveView pages or static pages.\n\n  It also handles navigation between LiveView and static pages seamlessly. So, you\n  don't have to worry about what type of page you're visiting. Just write the\n  tests from the user's perspective.\n\n  Thus, you can test a flow going from static to LiveView pages and back without\n  having to worry about the underlying implementation.\n\n  This is a sample flow:\n\n  ```elixir\n  test \"admin can create a user\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> click_link(\"Users\")\n    |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedan.com\")\n    |> click_button(\"Create\")\n    |> assert_has(\".user\", \"Aragorn\")\n  end\n  ```\n\n  Note that PhoenixTest does _not_ handle JavaScript. If you're looking for\n  something that supports JavaScript, take a look at\n  [Wallaby](https://hexdocs.pm/wallaby/readme.html).","ref":"PhoenixTest.html"},{"type":"function","title":"PhoenixTest.assert_has/2","doc":"Assert helper to ensure an element with given CSS selector is present.\n\nIt'll raise an error if no elements are found, but it will _not_ raise if more\nthan one matching element is found.\n\nIf you want to specify the content of the element, use `assert_has/3`.","ref":"PhoenixTest.html#assert_has/2"},{"type":"function","title":"PhoenixTest.assert_has/3","doc":"Assert helper to ensure an element with given CSS selector and `text` is\npresent.\n\nIt'll raise an error if no elements are found, but it will _not_ raise if more\nthan one matching element is found.","ref":"PhoenixTest.html#assert_has/3"},{"type":"function","title":"PhoenixTest.click_button/2","doc":"Perfoms action defined by button (and based on attributes present).\n\nThis can be used in a number of ways.","ref":"PhoenixTest.html#click_button/2"},{"type":"function","title":"Button with `phx-click` - PhoenixTest.click_button/2","doc":"If the button has a `phx-click` on it, it'll send the event to the LiveView.\n\n#","ref":"PhoenixTest.html#click_button/2-button-with-phx-click"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```heex\n Save \n```\n\n```elixir\nsession\n|> click_button(\"Save\") # <- will send \"save\" event to LiveView\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Button relying on Phoenix.HTML.js - PhoenixTest.click_button/2","doc":"If the button acts as a form via Phoenix.HTML's `data-method`, `data-to`, and\n`data-csrf`, this will emulate Phoenix.HTML.js and submit the form via data\nattributes.\n\nBut note that this _doesn't guarantee_ the JavaScript that handles form\nsubmissions via `data` attributes is loaded. The test emulates the behavior\nbut you must make sure the JavaScript is loaded.\n\nFor more on that, see https://hexdocs.pm/phoenix_html/Phoenix.HTML.html#module-javascript-library\n\n#","ref":"PhoenixTest.html#click_button/2-button-relying-on-phoenix-html-js"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```heex\n Delete \n```\n\n```elixir\nsession\n|> click_button(\"Delete\") # <- will submit form like Phoenix.HTML.js does\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Combined with `fill_form/3` - PhoenixTest.click_button/2","doc":"This function can be preceded by `fill_form` to fill out a form and\nsubsequently submit the form.\n\nNote that `fill_form/3` + `click_button/2` works for both static and live\npages.\n\n#","ref":"PhoenixTest.html#click_button/2-combined-with-fill_form-3"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```elixir\nsession\n|> fill_form(\"#user-form\", name: \"Aragorn\")\n|> click_button(\"Create\")\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"Single-button form - PhoenixTest.click_button/2","doc":"If `click_button/2` is used alone (without `phx-click`, `data-*` attrs, or\n`fill_form/3`), it is assumed it is a form with a single button (e.g.\n\"Delete\").\n\n#","ref":"PhoenixTest.html#click_button/2-single-button-form"},{"type":"function","title":"Example - PhoenixTest.click_button/2","doc":"```heex\n \n   \n   Delete \n \n```\n\n```elixir\nsession\n|> click_button(\"Delete\") # <- Triggers full form delete\n```","ref":"PhoenixTest.html#click_button/2-example"},{"type":"function","title":"PhoenixTest.click_button/3","doc":"Performs action defined by button with CSS selector.\n\nSee `click_button/2` for more details.","ref":"PhoenixTest.html#click_button/3"},{"type":"function","title":"PhoenixTest.click_link/2","doc":"Clicks a link with given text and performs the action.\n\nHere's how it handles different types of `a` tags:\n\n- With `href`: follows it to the next page\n- With `phx-click`: it'll send the event to the appropriate LiveView\n- With live redirect: it'll follow the live navigation to the next LiveView\n- With live patch: it'll patch the current LiveView","ref":"PhoenixTest.html#click_link/2"},{"type":"function","title":"Examples - PhoenixTest.click_link/2","doc":"```heex\n<.link href=\"/page/2\">Page 2</.link>\n<.link phx-click=\"next-page\">Next Page</.link>\n<.link navigate=\"next-liveview\">Next LiveView</.link>\n<.link patch=\"page/details\">Page Details</.link>\n```\n\n```elixir\nsession\n|> click_link(\"Page 2\") # <- follows to next page\n\nsession\n|> click_link(\"Next Page\") # <- sends \"next-page\" event to LiveView\n\nsession\n|> click_link(\"Next LiveView\") # <- follows to next LiveView\n\nsession\n|> click_link(\"Page Details\") # <- applies live patch\n```","ref":"PhoenixTest.html#click_link/2-examples"},{"type":"function","title":"Submitting forms - PhoenixTest.click_link/2","doc":"Phoenix allows for submitting forms on links via Phoenix.HTML's `data-method`,\n`data-to`, and `data-csrf`.\n\nWe can use `click_link` to emulate Phoenix.HTML.js and submit the\nform via data attributes.\n\nBut note that this _doesn't guarantee_ the JavaScript that handles form\nsubmissions via `data` attributes is loaded. The test emulates the behavior\nbut you must make sure the JavaScript is loaded.\n\nFor more on that, see https://hexdocs.pm/phoenix_html/Phoenix.HTML.html#module-javascript-library\n\n#","ref":"PhoenixTest.html#click_link/2-submitting-forms"},{"type":"function","title":"Example - PhoenixTest.click_link/2","doc":"```heex\n \n  Delete\n \n```\n\n```elixir\nsession\n|> click_link(\"Delete\") # <- will submit form like Phoenix.HTML.js does\n```","ref":"PhoenixTest.html#click_link/2-example"},{"type":"function","title":"PhoenixTest.click_link/3","doc":"Clicks a link with given CSS selector and text and performs the action.\nselector to target the link.\n\nSee `click_link/2` for more details.","ref":"PhoenixTest.html#click_link/3"},{"type":"function","title":"PhoenixTest.fill_form/3","doc":"Fills form data, validating that input fields are present.\n\nThis can be used by both static and live pages.\n\nIf the form is a LiveView form, and if the form has a `phx-change` attribute\ndefined, `fill_form/3` will trigger the `phx-change` event.\n\nThis can be followed by a `click_button/3` to submit the form.","ref":"PhoenixTest.html#fill_form/3"},{"type":"function","title":"Examples - PhoenixTest.fill_form/3","doc":"```elixir\nsession\n|> fill_form(\"#user-form\", name: \"Aragorn\")\n|> click_button(\"Create\")\n```\n\nIf your form has nested data -- for example, with an input such as ` ` -- you can pass a nested map as the last argument:\n\n```elixir\nsession\n|> fill_form(\"#user-form\", user: %{email: \"aragorn@dunedain.com\"})\n|> click_button(\"Create\")\n```","ref":"PhoenixTest.html#fill_form/3-examples"},{"type":"function","title":"PhoenixTest.open_browser/1","doc":"Open the default browser to display current HTML of `session`.","ref":"PhoenixTest.html#open_browser/1"},{"type":"function","title":"Examples - PhoenixTest.open_browser/1","doc":"```elixir\nsession\n|> visit(\"/\")\n|> open_browser()\n|> submit_form(\"#user-form\", name: \"Aragorn\")\n```","ref":"PhoenixTest.html#open_browser/1-examples"},{"type":"function","title":"PhoenixTest.refute_has/2","doc":"Opposite of `assert_has/2` helper. Verifies that element with\ngiven CSS selector is _not_ present.\n\nIt'll raise an error if any elements that match selector are found.\n\nIf you want to specify the content of the element, use `refute_has/3`.","ref":"PhoenixTest.html#refute_has/2"},{"type":"function","title":"PhoenixTest.refute_has/3","doc":"Opposite of `assert_has/3` helper. Verifies that element with\ngiven CSS selector and `text` is _not_ present.\n\nIt'll raise an error if any elements that match selector and text are found.","ref":"PhoenixTest.html#refute_has/3"},{"type":"function","title":"PhoenixTest.submit_form/3","doc":"Submits form in the same way one would do by pressing ` `.\n\n_Note that this does not validate presence of the submit button._\n\nIn the case of LiveView forms, it'll submit the form with LiveView's\n`phx-submit` event.\n\nIf it's a static form, this is equivalent to filling the form and submitting\nit with the form's `method` and to the form's `action`.\n\nNote: if your form has a submit button, it's recommended you test with\n`fill_form/3` + `click_button/2` instead.","ref":"PhoenixTest.html#submit_form/3"},{"type":"function","title":"Examples - PhoenixTest.submit_form/3","doc":"```elixir\nsession\n|> submit_form(\"#user-form\", name: \"Aragorn\")\n```\n\nIf your form has nested data -- for example, with an input such as ` ` -- you can pass a nested map as the last argument:\n\n```elixir\nsession\n|> submit_form(\"#user-form\", user: %{email: \"aragorn@dunedain.com\"})\n```","ref":"PhoenixTest.html#submit_form/3-examples"},{"type":"function","title":"PhoenixTest.visit/2","doc":"Entrypoint to create a session.\n\n`visit/2` takes a `Plug.Conn` struct and the path to visit.\n\nIt returns a `session` which the rest of the `PhoenixTest` functions can use.\n\nNote that `visit/2` is smart enough to know if the page you're visiting is a\nLiveView or a static view. You don't need to worry about which type of page\nyou're visiting.","ref":"PhoenixTest.html#visit/2"},{"type":"extras","title":"Introduction","doc":"# PhoenixTest\n\n[![Module Version](https://img.shields.io/hexpm/v/phoenix_test.svg)](https://hex.pm/packages/phoenix_test/)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/phoenix_test/)\n[![License](https://img.shields.io/hexpm/l/phoenix_test.svg)](https://github.com/germsvel/phoenix_test/blob/main/LICENSE)\n\nPhoenixTest provides a unified way of writing feature tests -- regardless of\nwhether you're testing LiveView pages or static (non-LiveView) pages.\n\nIt also handles navigation between LiveView and static pages seamlessly. So, you\ndon't have to worry about what type of page you're visiting. Just write the\ntests from the user's perspective.\n\nThus, you can test a flow going from static to LiveView pages and back without\nhaving to worry about the underlying implementation.\n\nThis is a sample flow:\n\n```elixir\ntest \"admin can create a user\", %{conn: conn} do\n  conn\n  |> visit(\"/\")\n  |> click_link(\"Users\")\n  |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedain.com\")\n  |> click_button(\"Create\")\n  |> assert_has(\".user\", \"Aragorn\")\nend\n```\n\nNote that PhoenixTest does not handle JavaScript. If you're looking for\nsomething that supports JavaScript, take a look at\n[Wallaby](https://hexdocs.pm/wallaby/readme.html).","ref":"readme.html"},{"type":"extras","title":"Setup - Introduction","doc":"PhoenixTest requires Phoenix `1.7+` and LiveView `0.20+`. It may work with\nearlier versions, but I have not tested that.\n\n#","ref":"readme.html#setup"},{"type":"extras","title":"Installation - Introduction","doc":"Add `phoenix_test` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:phoenix_test, \"~> 0.2.9\", only: :test, runtime: false}\n  ]\nend\n```\n\n#","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - Introduction","doc":"In `config/test.exs` specify the endpoint to be used for routing requests:\n\n```elixir\nconfig :phoenix_test, :endpoint, MyAppWeb.Endpoint\n```\n\n#","ref":"readme.html#configuration"},{"type":"extras","title":"Adding a `FeatureCase` - Introduction","doc":"`PhoenixTest` helpers can be included via `import PhoenixTest`.\n\nBut since each test needs a `conn` struct to get started, you'll likely want to\nset up a few things before that.\n\nTo make that easier, it's helpful to create a `FeatureCase` module that can be\nused from your tests (replace `MyApp` with your app's name):\n\n```elixir\ndefmodule MyAppWeb.FeatureCase do\n  @moduledoc \"\"\"\n  This module defines the test case to be used by tests that require setting up\n  a connection to test feature tests.\n\n  Such tests rely on `PhoenixTest` and also import other functionality to\n  make it easier to build common data structures and interact with pages.\n\n  Finally, if the test case interacts with the database, we enable the SQL\n  sandbox, so changes done to the database are reverted at the end of every\n  test. If you are using PostgreSQL, you can even run database tests\n  asynchronously by setting `use MyAppWeb.FeatureCase, async: true`, although\n  this option is not recommended for other databases.\n  \"\"\"\n\n  use ExUnit.CaseTemplate\n\n  using do\n    quote do\n      use MyAppWeb, :verified_routes\n\n      import MyAppWeb.FeatureCase\n\n      import PhoenixTest\n    end\n  end\n\n  setup tags do\n    pid = Ecto.Adapters.SQL.Sandbox.start_owner!(MyApp.Repo, shared: not tags[:async])\n    on_exit(fn -> Ecto.Adapters.SQL.Sandbox.stop_owner(pid) end)\n\n    {:ok, conn: Phoenix.ConnTest.build_conn()}\n  end\nend\n```\n\nNote that we assume your Phoenix project is using Ecto and its phenomenal\n`SQL.Sandbox`. If it doesn't, feel free to remove the `SQL.Sandbox` code above.","ref":"readme.html#adding-a-featurecase"},{"type":"extras","title":"Usage - Introduction","doc":"Now, you can create your tests like this:\n\n```elixir\n# test/my_app_web/features/admin_can_create_user_test.exs\n\ndefmodule MyAppWeb.AdminCanCreateUserTest do\n  use MyAppWeb.FeatureCase, async: true\n\n  test \"admin can create user\", %{conn: conn} do\n    conn\n    |> visit(\"/\")\n    |> click_link(\"Users\")\n    |> fill_form(\"#user-form\", name: \"Aragorn\", email: \"aragorn@dunedain.com\")\n    |> click_button(\"Create\")\n    |> assert_has(\".user\", \"Aragorn\")\n  end\n```\n\nFor full documentation, take a look at [PhoenixTest module docs](https://hexdocs.pm/phoenix_test/PhoenixTest.html).","ref":"readme.html#usage"},{"type":"extras","title":"Why PhoenixTest? - Introduction","doc":"#","ref":"readme.html#why-phoenixtest"},{"type":"extras","title":"A unified way of writing feature tests - Introduction","doc":"With the advent of LiveView, I find myself writing less and less JavaScript.\n\nSure, there are sprinkles of it here and there, and there’s always the\noccasional need for something more.\n\nBut for the most part:\n\n- If I’m going to build a page that needs interactivity, I use LiveView.\n- If I’m going to write a static page, I use regular controllers + views/HTML\n  modules.\n\nThe problem is that LiveView pages and static pages have _vastly different_\ntesting strategies.\n\nIf we use LiveView, we have a good set of helpers.\n\n```elixir\n{:ok, view, _html} = live(conn, ~p\"/\")\n\nhtml =\n  view\n  |> element(\"#greet-guest\")\n  |> render_click()\n\nassert html =~ \"Hello, guest!\"\n```\n\nBut if we're testing a static page, we have to resort to controller testing:\n\n```elixir\nconn = get(conn, ~p\"/greet_page\")\n\nassert html_response(conn, 200) =~ \"Hello, guest!\"\n```\n\nThat means we don’t have ways of interacting with static pages at all!\n\nWhat if we want to submit a form or click a link? And what if a click takes us\nfrom a LiveView to a static view or vice versa?\n\nInstead, I'd like to have a unified way of testing Phoenix apps -- regardless of\nwhether we're testing LiveView pages or static pages.\n\n#","ref":"readme.html#a-unified-way-of-writing-feature-tests"},{"type":"extras","title":"Improved assertions - Introduction","doc":"And then there's the problem of assertions.\n\nBecause LiveView and controller tests use `=~` for assertions, the error\nmessages aren't very helpful when assertions fail.\n\nAfter all, we’re just comparing two blobs of text – and trust me, HTML pages can\nget very large and hard to read as a blob of text in your terminal.\n\nLiveView tries to help with its `has_element?/3` helper, which allows us to\ntarget elements by CSS selectors and text.\n\nUnfortunately, it still doesn't provide the best errors.\n\n`has_element?/3` only tells us what was passed into the function. It doesn't\ngive us a clue as to what else might've been on the page – maybe we just made a\nsmall typo and we have no idea!\n\nAnd that's where `PhoenixTest` comes in! A unified way of writing feature tests\nand improved assertions where they're needed!","ref":"readme.html#improved-assertions"},{"type":"extras","title":"What do you mean by \"static\" pages? - Introduction","doc":"We use the term _static_ as compared to LiveView pages. Thus, in PhoenixTest's\nterminology static pages are what is typically known as dynamic, server-rendered\npages -- pages that were normal prior to the advent of LiveView and which are\nsometimes called \"dead\" views. Thus, we do not mean _static_ in the sense that\nstatic-site generators (such as Jekyll, Gatsby, etc.) mean it.","ref":"readme.html#what-do-you-mean-by-static-pages"},{"type":"extras","title":"Sponsors - Introduction","doc":"Made possible thanks to:\n\n \n   \n     \n     Elixir Streams","ref":"readme.html#sponsors"},{"type":"extras","title":"Changelog","doc":"Changelog\n=========\n\nNoteworthy changes are included here. For a full version of changes, see git\nhistory.\n\nTo see dates a version was published, see the [hex package\npage](https://hex.pm/packages/phoenix_test)","ref":"changelog.html"},{"type":"extras","title":"0.2.9 - Changelog","doc":"#","ref":"changelog.html#0-2-9"},{"type":"extras","title":"Improvements - Changelog","doc":"- Adds `assert_has/2` and `refute_has/2`. Commits [3756a47] and [9709b7a].\n- Follows redirect on `visit/2`. Commit [b4f49be]\n\n#","ref":"changelog.html#improvements"},{"type":"extras","title":"Fixes - Changelog","doc":"- Do not always assume `submit_button` after `fill_form` is for submitting the\n  form. Commit [e193a07]\n\n[3756a47]: https://github.com/germsvel/phoenix_test/commit/3756a47\n[9709b7a]: https://github.com/germsvel/phoenix_test/commit/9709b7a\n[b4f49be]: https://github.com/germsvel/phoenix_test/commit/b4f49be\n[e193a07]: https://github.com/germsvel/phoenix_test/commit/e193a07","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.8 - Changelog","doc":"#","ref":"changelog.html#0-2-8"},{"type":"extras","title":"Added - Changelog","doc":"- Adds WSL2 for `open_browser/2`. Commit [b852014].\n- Relax Elixir version requirement to 1.15. Commit [3cb9586]\n- Support visiting non-200 pages in Static implementation. Commit [4964ab2].\n\n[b852014]: https://github.com/germsvel/phoenix_test/commit/b852014\n[3cb9586]: https://github.com/germsvel/phoenix_test/commit/3cb9586\n[4964ab2]: https://github.com/germsvel/phoenix_test/commit/4964ab2","ref":"changelog.html#added"},{"type":"extras","title":"0.2.7 - Changelog","doc":"#","ref":"changelog.html#0-2-7"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fixes `open_browser/1` not existing. Commit [7407b19].\n\n[7407b19]: https://github.com/germsvel/phoenix_test/commit/7407b19","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.6 - Changelog","doc":"#","ref":"changelog.html#0-2-6"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `open_browser/1` function to both Live and Static implementations.\n  Commit [b9d8347].\n- Handle forms that use `data` attributes and `Phoenix.HTML.js` to submit forms.\n  Commit [d699792].\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Correctly handles forms that PUT/DELETE (through hidden inputs). Commit\n  [3efa4c0].\n\n[b9d8347]: https://github.com/germsvel/phoenix_test/commit/b9d8347\n[3efa4c0]: https://github.com/germsvel/phoenix_test/commit/3efa4c0\n[d699792]: https://github.com/germsvel/phoenix_test/commit/d699792","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.5 - Changelog","doc":"#","ref":"changelog.html#0-2-5"},{"type":"extras","title":"Added - Changelog","doc":"- Introduce ability to assert and refute on page title. Commit [8552ec7].\n- Handle regular form submission from LiveView pages when using `submit_form`.\n  Commit [fc4d3ef].\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Improve Live validation of form fields to properly handle nested fields in\n  forms. Commits [180dc0d] and [c275c0c].\n\n[8552ec7]: https://github.com/germsvel/phoenix_test/commit/8552ec7\n[fc4d3ef]: https://github.com/germsvel/phoenix_test/commit/fc4d3ef\n[180dc0d]: https://github.com/germsvel/phoenix_test/commit/180dc0d\n[c275c0c]: https://github.com/germsvel/phoenix_test/commit/c275c0c","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.4 - Changelog","doc":"#","ref":"changelog.html#0-2-4"},{"type":"extras","title":"Added - Changelog","doc":"- Handle form redirects from static pages. Commit\n  [4c39920](https://github.com/germsvel/phoenix_test/commit/4c39920)\n- Handle regular form submission from LiveView pages with `fill_form` +\n  `click_button`. Commit [fe755de](https://github.com/germsvel/phoenix_test/commit/fe755de)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Use Html.raw/1 for more errors to handle nested buttons.\n  [82e7415](https://github.com/germsvel/phoenix_test/commit/82e7415)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.3 - Changelog","doc":"#","ref":"changelog.html#0-2-3"},{"type":"extras","title":"Added - Changelog","doc":"- Handle form redirects (to Live and static pages) from Live pages. Commit\n  [531e5e9](https://github.com/germsvel/phoenix_test/commit/531e5e9)\n- Expand documentation on nested forms. Commit\n  [6809389](https://github.com/germsvel/phoenix_test/commit/6809389)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Allow multiple matching elements in `assert_has`. Commit\n  [ac0e167](https://github.com/germsvel/phoenix_test/commit/ac0e167)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.2 - Changelog","doc":"#","ref":"changelog.html#0-2-2"},{"type":"extras","title":"Added - Changelog","doc":"- Raise `AssertionError` instead of `RuntimeError` in assertions for more\n  consistent ExUnit error messages. Commit\n  [117bc59](https://github.com/germsvel/phoenix_test/commit/117bc59)\n- Update `fill_form` to handle that aren't direct children of the `form`\n  element. Commit\n  [46d6229](https://github.com/germsvel/phoenix_test/commit/46d6229)","ref":"changelog.html#added"},{"type":"extras","title":"0.2.1 - Changelog","doc":"#","ref":"changelog.html#0-2-1"},{"type":"extras","title":"Added - Changelog","doc":"- Improve printing of complex nested content in assertions. Commit\n  [7151834](https://github.com/germsvel/phoenix_test/commit/7151834)\n- Better error messages in forms when multiple submit buttons/inputs are found.\n  Commit [82492c6](https://github.com/germsvel/phoenix_test/commit/82492c6)\n\n#","ref":"changelog.html#added"},{"type":"extras","title":"Fixes - Changelog","doc":"- Allow using `refute_has` in pipes in the same way `assert_has` already works.\n  Commit [0484979](https://github.com/germsvel/phoenix_test/commit/0484979)","ref":"changelog.html#fixes"},{"type":"extras","title":"0.2.0 - Changelog","doc":"#","ref":"changelog.html#0-2-0"},{"type":"extras","title":"Breaking changes - Changelog","doc":"- Update our static implementation to raise when we find many elements. That\n  brings it in line with how our LiveView implementation works. Commit\n  [daa4dca](https://github.com/germsvel/phoenix_test/commit/daa4dca)\n\n#","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Improved - Changelog","doc":"- Improves error messages in assertions. Commit [c995fc1](https://github.com/germsvel/phoenix_test/commit/c995fc1)","ref":"changelog.html#improved"},{"type":"extras","title":"0.1.1 - Changelog","doc":"#","ref":"changelog.html#0-1-1"},{"type":"extras","title":"Added - Changelog","doc":"- Adds `click_link/3` and `click_button/3` which allow for specifying a CSS\n  selector. Commit [c7401b6](https://github.com/germsvel/phoenix_test/commit/c7401b6).","ref":"changelog.html#added"},{"type":"extras","title":"0.1.0 - Changelog","doc":"- Initial version of the library.","ref":"changelog.html#0-1-0"}],"content_type":"text/markdown"}